package dto

import (
	"go-admin/app/admin/models"
	"go-admin/common/dto"
	common "go-admin/common/models"
)

type OtIframGetPageReq struct {
	dto.Pagination `search:"-"`
	Url            string `form:"url"  search:"type:exact;column:url;table:ot_iframe" comment:"iframe地址"`
	PageTag        string `form:"pageTag"  search:"type:exact;column:page_tag;table:ot_iframe" comment:"页面标识"`
	AppId          int64  `form:"appId"  search:"type:exact;column:app_id;table:ot_iframe" comment:"AppID"`
	Overall        int64  `form:"overall"  search:"type:exact;column:overall;table:ot_iframe" comment:"Overall 全局可用"`
	OtIframOrder
}

type OtIframOrder struct {
	Id        string `form:"idOrder"  search:"type:order;column:id;table:ot_iframe"`
	CreatedAt string `form:"createdAtOrder"  search:"type:order;column:created_at;table:ot_iframe"`
	UpdatedAt string `form:"updatedAtOrder"  search:"type:order;column:updated_at;table:ot_iframe"`
	DeletedAt string `form:"deletedAtOrder"  search:"type:order;column:deleted_at;table:ot_iframe"`
	CreateBy  string `form:"createByOrder"  search:"type:order;column:create_by;table:ot_iframe"`
	UpdateBy  string `form:"updateByOrder"  search:"type:order;column:update_by;table:ot_iframe"`
	Desc      string `form:"descOrder"  search:"type:order;column:desc;table:ot_iframe"`
	Url       string `form:"urlOrder"  search:"type:order;column:url;table:ot_iframe"`
	PageTag   string `form:"pageTagOrder"  search:"type:order;column:page_tag;table:ot_iframe"`
	AppId     string `form:"appIdOrder"  search:"type:order;column:app_id;table:ot_iframe"`
	Overall   string `form:"overallOrder"  search:"type:order;column:overall;table:ot_iframe"`
}

func (m *OtIframGetPageReq) GetNeedSearch() interface{} {
	return *m
}

type OtIframInsertReq struct {
	Id      int    `json:"-" comment:""` //
	Desc    string `json:"desc" comment:""`
	Url     string `json:"url" comment:"iframe地址"`
	PageTag string `json:"pageTag" comment:"页面标识"`
	AppId   int64  `json:"appId" comment:"AppID"`
	Overall int64  `json:"overall" comment:"Overall"`
	common.ControlBy
}

func (s *OtIframInsertReq) Generate(model *models.OtIfram) {
	if s.Id == 0 {
		model.Model = common.Model{Id: s.Id}
	}
	model.CreateBy = s.CreateBy // 添加这而，需要记录是被谁创建的
	model.Desc = s.Desc
	model.Url = s.Url
	model.PageTag = s.PageTag
	model.AppId = s.AppId
	model.Overall = s.Overall
}

func (s *OtIframInsertReq) GetId() interface{} {
	return s.Id
}

type OtIframUpdateReq struct {
	Id      int    `uri:"id" comment:""` //
	Desc    string `json:"desc" comment:""`
	Url     string `json:"url" comment:"iframe地址"`
	PageTag string `json:"pageTag" comment:"页面标识"`
	AppId   int64  `json:"appId" comment:"AppID"`
	Overall int64  `json:"overall" comment:"Overall"`
	common.ControlBy
}

func (s *OtIframUpdateReq) Generate(model *models.OtIfram) {
	if s.Id == 0 {
		model.Model = common.Model{Id: s.Id}
	}
	model.UpdateBy = s.UpdateBy // 添加这而，需要记录是被谁更新的
	model.Desc = s.Desc
	model.Url = s.Url
	model.PageTag = s.PageTag
	model.AppId = s.AppId
	model.Overall = s.Overall
}

func (s *OtIframUpdateReq) GetId() interface{} {
	return s.Id
}

// OtIframGetReq 功能获取请求参数
type OtIframGetReq struct {
	Id int `uri:"id"`
}

func (s *OtIframGetReq) GetId() interface{} {
	return s.Id
}

// OtIframDeleteReq 功能删除请求参数
type OtIframDeleteReq struct {
	Ids []int `json:"ids"`
}

func (s *OtIframDeleteReq) GetId() interface{} {
	return s.Ids
}
