package service

import (
	"errors"

	"github.com/go-admin-team/go-admin-core/sdk/service"
	"gorm.io/gorm"

	"go-admin/app/admin/models"
	"go-admin/app/admin/service/dto"
	"go-admin/common/actions"
	cDto "go-admin/common/dto"
)

type OtAppDashboard struct {
	service.Service
}

// GetPage 获取OtAppDashboard列表
func (e *OtAppDashboard) GetPage(c *dto.OtAppDashboardGetPageReq, p *actions.DataPermission, list *[]models.OtAppDashboard, count *int64) error {
	var err error
	var data models.OtAppDashboard

	err = e.Orm.Model(&data).
		Scopes(
			cDto.MakeCondition(c.GetNeedSearch()),
			cDto.Paginate(c.GetPageSize(), c.GetPageIndex()),
			actions.Permission(data.TableName(), p),
		).
		Find(list).Limit(-1).Offset(-1).
		Count(count).Error
	if err != nil {
		e.Log.Errorf("OtAppDashboardService GetPage error:%s \r\n", err)
		return err
	}
	return nil
}

// Get 获取OtAppDashboard对象
func (e *OtAppDashboard) Get(d *dto.OtAppDashboardGetReq, p *actions.DataPermission, model *models.OtAppDashboard) error {
	var data models.OtAppDashboard

	err := e.Orm.Model(&data).
		Scopes(
			actions.Permission(data.TableName(), p),
		).
		First(model, d.GetId()).Error
	if err != nil && errors.Is(err, gorm.ErrRecordNotFound) {
		err = errors.New("查看对象不存在或无权查看")
		e.Log.Errorf("Service GetOtAppDashboard error:%s \r\n", err)
		return err
	}
	if err != nil {
		e.Log.Errorf("db error:%s", err)
		return err
	}
	return nil
}

// Insert 创建OtAppDashboard对象
func (e *OtAppDashboard) Insert(c *dto.OtAppDashboardInsertReq) error {
	var err error
	var data models.OtAppDashboard
	c.Generate(&data)
	err = e.Orm.Create(&data).Error
	if err != nil {
		e.Log.Errorf("OtAppDashboardService Insert error:%s \r\n", err)
		return err
	}
	return nil
}

// Update 修改OtAppDashboard对象
func (e *OtAppDashboard) Update(c *dto.OtAppDashboardUpdateReq, p *actions.DataPermission) error {
	var err error
	var data = models.OtAppDashboard{}
	e.Orm.Scopes(
		actions.Permission(data.TableName(), p),
	).First(&data, c.GetId())
	c.Generate(&data)

	db := e.Orm.Save(&data)
	if err = db.Error; err != nil {
		e.Log.Errorf("OtAppDashboardService Save error:%s \r\n", err)
		return err
	}
	if db.RowsAffected == 0 {
		return errors.New("无权更新该数据")
	}
	return nil
}

// Remove 删除OtAppDashboard
func (e *OtAppDashboard) Remove(d *dto.OtAppDashboardDeleteReq, p *actions.DataPermission) error {
	var data models.OtAppDashboard

	db := e.Orm.Model(&data).
		Scopes(
			actions.Permission(data.TableName(), p),
		).Delete(&data, d.GetId())
	if err := db.Error; err != nil {
		e.Log.Errorf("Service RemoveOtAppDashboard error:%s \r\n", err)
		return err
	}
	if db.RowsAffected == 0 {
		return errors.New("无权删除该数据")
	}
	return nil
}

func (e *OtAppDashboard) GetByAppId(m *models.OtAppDashboard) error {
	err := e.Orm.Model(m).
		Where("app_id = ?", m.AppId).
		First(m).Error
	if err != nil {
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return nil
		} else {
			e.Log.Errorf("db error:%s", err)
			return err
		}
	}
	return nil
}
