package service

import (
	"errors"
	"fmt"

	"github.com/go-admin-team/go-admin-core/sdk/service"
	"gorm.io/gorm"

	"go-admin/app/admin/models"
	"go-admin/app/admin/service/dto"
	"go-admin/common/actions"
	cDto "go-admin/common/dto"
)

type OtPlugins struct {
	service.Service
}

// GetPage 获取OtPlugins列表
func (e *OtPlugins) GetPage(c *dto.OtPluginsGetPageReq, p *actions.DataPermission, list *[]models.OtPlugins, count *int64) error {
	var err error
	var data models.OtPlugins

	err = e.Orm.Model(&data).
		Scopes(
			cDto.MakeCondition(c.GetNeedSearch()),
			cDto.Paginate(c.GetPageSize(), c.GetPageIndex()),
			actions.Permission(data.TableName(), p),
		).
		Find(list).Limit(-1).Offset(-1).
		Count(count).Error
	if err != nil {
		e.Log.Errorf("OtPluginsService GetPage error:%s \r\n", err)
		return err
	}
	return nil
}

// Get 获取OtPlugins对象
func (e *OtPlugins) Get(d *dto.OtPluginsGetReq, p *actions.DataPermission, model *models.OtPlugins) error {
	var data models.OtPlugins

	err := e.Orm.Model(&data).
		Scopes(
			actions.Permission(data.TableName(), p),
		).
		First(model, d.GetId()).Error
	if err != nil && errors.Is(err, gorm.ErrRecordNotFound) {
		err = errors.New("查看对象不存在或无权查看")
		e.Log.Errorf("Service GetOtPlugins error:%s \r\n", err)
		return err
	}
	if err != nil {
		e.Log.Errorf("db error:%s", err)
		return err
	}
	return nil
}

// Insert 创建OtPlugins对象
func (e *OtPlugins) Insert(c *dto.OtPluginsInsertReq) error {
	var err error
	var data models.OtPlugins
	c.Generate(&data)
	err = e.Orm.Create(&data).Error
	if err != nil {
		e.Log.Errorf("OtPluginsService Insert error:%s \r\n", err)
		return err
	}
	return nil
}

// Update 修改OtPlugins对象
func (e *OtPlugins) Update(c *dto.OtPluginsUpdateReq, p *actions.DataPermission) error {
	var err error
	var data = models.OtPlugins{}
	e.Orm.Scopes(
		actions.Permission(data.TableName(), p),
	).First(&data, c.GetId())
	c.Generate(&data)

	db := e.Orm.Save(&data)
	if err = db.Error; err != nil {
		e.Log.Errorf("OtPluginsService Save error:%s \r\n", err)
		return err
	}
	if db.RowsAffected == 0 {
		return errors.New("无权更新该数据")
	}
	return nil
}

// Remove 删除OtPlugins
func (e *OtPlugins) Remove(d *dto.OtPluginsDeleteReq, p *actions.DataPermission) error {
	var data models.OtPlugins

	db := e.Orm.Model(&data).
		Scopes(
			actions.Permission(data.TableName(), p),
		).Delete(&data, d.GetId())
	if err := db.Error; err != nil {
		e.Log.Errorf("Service RemoveOtPlugins error:%s \r\n", err)
		return err
	}
	if db.RowsAffected == 0 {
		return errors.New("无权删除该数据")
	}
	return nil
}

func (e *OtPlugins) GetPluginByAlias(m *models.OtPlugins) error {
	fmt.Println("modelplugin: ", m.Alias)
	err := e.Orm.Model(m).
		Where("alias = ?", m.Alias).
		First(m).Error
	if err != nil {
		if errors.Is(err, gorm.ErrRecordNotFound) {
			err = errors.New("查看对象不存在")
			e.Log.Errorf("Service GetOtPluginByAlias error:%s \r\n", err)
			return err
		} else {
			e.Log.Errorf("db error:%s", err)
			return err
		}
	}
	return nil
}
